import sys
import json
from datetime import datetime

import requests


class LogEntry:
    """Class describing a connection attempt"""

    def __init__(self: 'LogEntry', log_entry: str) -> None:
        self.date = datetime.strptime(log_entry[1:20], '%Y-%m-%d %H:%M:%S')
        self.host = log_entry.split('[')[2].split(']')[0]
        self.failed = 'FAIL' in log_entry


def printOutages(config: dict) -> None:
    """Print outages from a log file"""

    with open(config['log'], 'r') as file:
        log = list(map(lambda x: LogEntry(x), file.readlines()))

    # cluster log entries by host
    log_by_host = {}
    for entry in log:
        if entry.host not in log_by_host:
            log_by_host[entry.host] = [entry]
        else:
            log_by_host[entry.host].append(entry)

    # cluster fails to outages
    # we consider the time of subsequent failed connection attemts to the same host an outage
    outages: list = []
    last_failed = False
    for entries in log_by_host.values():
        for entry in entries:
            if entry.failed and last_failed:
                outages[-1].append(entry)
            if entry.failed and not last_failed:
                outages.append([entry])
                last_failed = True
            if not entry.failed and last_failed:
                last_failed = False

    # print the outages by date of first fail
    for outage in sorted(outages, key=lambda x: x[0].date):

        # get duration of outage in hours and minutes
        outage_duration = round(
            (outage[-1].date - outage[0].date).seconds / 60)
        # continue if outage is shorter than the minimum duration
        if outage_duration < config['outages']:
            continue
        hours = outage_duration // 60
        minutes = outage_duration - (hours * 60)
        host = outage[0].host

        # Outputs outages in the form of "Outage at: 2023-19-01 06:29:01 lasting for 2 Hours and 39 Minutes"
        print(f"[{outage[0].date}][{host}] lasting for {'{} Hours and '.format(hours) if hours >= 1 else ''}{minutes} Minutes")


def isOnline(host: str, timeout: int) -> bool:
    """Check if connection to a host can be established"""

    try:
        requests.head(f"https://{host}", timeout=timeout)
        return True
    except requests.ConnectionError:
        return False


def log(host: int, timeout: int, no_stdout: bool, log_path: None | str = None) -> None:
    """Log the connection status of a host"""

    logline = f"[{datetime.now().strftime('%Y-%m-%d %H:%M:%S')}][{host}][{'OK' if isOnline(host, timeout) else 'FAIL'}]\n"

    if log_path is not None:
        with open(log_path, 'a') as file:
            file.write(logline)

    if not no_stdout:
        print(logline)


def parseArgs(args: list[str]):
    import argparse

    parser = argparse.ArgumentParser(
        prog='outage_detector', description='Log outages and print statistics.', formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    subparsers = parser.add_subparsers()

    # Arguments for the log command
    parser_log = subparsers.add_parser(
        'log', help='Log the connection status.', formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser_log.add_argument(
        '--path', type=str, default=None,
        help='Path to a the file where results should be appended to. Creates the file if it does not exist.'
    )
    parser_log.add_argument(
        '--host', type=str, default='1.1.1.1',
        help='IP or hostname of a server to query.'
    )
    parser_log.add_argument(
        '--timeout', type=int, default=2,
        help='Set the timeout in seconds to use for the connection test.'
    )
    parser_log.add_argument(
        '--no-stdout', action='store_true', default=argparse.SUPPRESS,
        help='A flag that disables printing to stdout.'
    )

    # Arguments for the outages command
    parser_outages = subparsers.add_parser(
        'outages', help='Print outages', formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser_outages.add_argument(
        '--log-path', type=str, default='./connection.log',
        help='Path to a file with connection logs.'
    )
    parser_outages.add_argument(
        '--time', type=int, default=3,
        help='Minimum duration for a connection loss to be considered an outage.'
    )

    print(parser.parse_args(args))


def main():
    from sys import argv

    parseArgs(argv[1:])
    # config, mode = loadConfig()
    # if mode == 'log':
    #     log(config)
    # elif mode == 'outages':
    #     printOutages(config)


if __name__ == '__main__':
    main()
